# coding=utf-8

"""
Created on 2015年11月4日

@author: thomas.ning
"""
from easyuiautomator.common.log import Logger
from easyuiautomator.driver.common.keyevent import KeyCode
from easyuiautomator.driver.common.setting_action import SettingAction
from easyuiautomator.common.exceptions import DeviceNotFound
import subprocess,tempfile,base64,os,time,re


class ADB(object):
    """
    adb provides the connection and basic actions for Android Device
    """ 
    
    def __init__(self, serial=None, restart=True, server_host=None, server_port=None):
        self.logger = Logger.getLogger()
        self.server_host = str(server_host if server_host else '127.0.0.1')
        self.server_port = str(server_port if server_port else '5037')
        self.serial = None
        self.restart = restart # after adb command execution failed, Whether to restart
        self.sdk_version = 0
        serial = serial or os.getenv('ANDROID_SERIAL')
        if serial is None:
            self.serial = self.getConnectedDevices()[0]['udid']
            self.logger.debug("bind device on serial " + self.serial)
        else:
            self.serial = serial
        
    def setRestartMode(self,flag):
        '''true:adb server重启，false: adb server不重启'''
        self.restart = flag

    def exec_adb(self, cmd):
        '''
        Execute adb command,if something wrong,adb will restart one time.
        while adb command failed or timeout ,it will break and raise Error.
                        执行ADB命令，如果出错，将会重启ADB服务，如果仍然报错或者超时，命令将会中断并抛出异常
        :Args:
         - cmd - command
        :Usage:
            exec_adb('device')
        :Return:
            error : exec_adb('device')
        '''
        out = self.cmd(cmd).communicate()[0]
        lines = []
        for line in out.strip().splitlines():
            if line.strip() == "":
                continue
            if 'not found' in line and 'system/bin/sh' not in line:
                if self._adbRecovery() is True:
                    return self._exec_adb_retry(cmd)
                else:
                    raise DeviceNotFound("[%s] %s"%(self.serial,line)) 
            lines.append(str.strip(line))
        return lines

    def _exec_adb_retry(self, cmd):
        self.logger.debug("{ %s }Retry Adb Command %s"%(self.serial,cmd))
        out = self.cmd(cmd).communicate()[0]
        lines = []
        for line in out.strip().splitlines():
            if line.strip() == "":
                continue
            if 'not found' in line and 'system/bin/sh' not in line:
                # case错误停止,异常处理
                raise DeviceNotFound("[%s] %s"%(self.serial,line))
            lines.append(str.strip(line))
        return lines

    def _adbRecovery(self):
        ''' Restart adb server three times.重启ADB服务3次'''
        self.logger.debug("adb restart")
        for tmp in range(3):
            self.logger.debug("Restart Adb Server %d" % (tmp + 1))
            if self.restart:
                os.system("adb kill-server")
                time.sleep(2)
                os.system("adb start-server")
            time.sleep(5)
            if tmp == 2:
                status_device = self.adbDevices()
                if status_device == True:
                    return True
                elif status_device == "offline":
                    self.logger.debug("%s device offline" %self.serial)
                    flag = self._pause_device()
                    if flag:
                        return True
                    raise DeviceNotFound("[%s] offline"%self.serial)
                else:
                    raise DeviceNotFound("[%s] not connected device"%self.serial)
            else: 
                if self._getConnectedDevices() == True:
                    return True
    
    def _pause_device(self, timeout=20):
        '''pause device 2 minutes, wait server interrupt adbd, then reconnect''' 
        time_unit = 5
        while time_unit <  timeout:
            time.sleep(5)
            time_unit += 5
            if self.adbDevices() == True:
                return True
        return False
            
    def adbDevices(self):
        out = self.raw_cmd('devices').communicate()[0]
        lines = [str(line).strip() for line in out.strip().splitlines()]
        if 'error' in lines:
            return False
        for line in lines:
            if line != "" \
            and "List of devices" not in line\
            and '* daemon' not in line\
            and 'offline' not in line:
                if self.serial is None:
                    return True
                elif self.serial in line:
                    return True
            if self.serial\
            and self.serial in line\
            and 'offline' in line:
                return "offline"
        return False 
    
    def _getConnectedDevices(self):
        '''by given serial or not given, Judgment connected device '''
        out = self.raw_cmd('devices').communicate()[0]
        lines = [str(line).strip() for line in out.strip().splitlines()]
        if 'error' in lines:
            return False
        for line in lines:
            if line != "" and "List of devices" not in line\
                    and '* daemon' not in line and 'offline' not in line:
                if self.serial is None:
                    return True
                elif self.serial in line:
                    return True
        return False
    
    def getConnectedDevices(self):
        '''Get connected devices and return it'''
        self.logger.debug("Get connect devices")
        lines = self.exec_adb('devices')
        devices = []
        if 'error' in lines:
            self.logger.error(lines)
            if self._adbRecovery() == True:
                lines = self._exec_adb_retry('devices')
                return self._deal_devicelist(lines)
            else:
                raise DeviceNotFound("not connected device")
        else:
            devices = self._deal_devicelist(lines)
            if len(devices) == 0:
                raise DeviceNotFound("not connected device")
        self.logger.debug("device list " + str(devices))
        return devices
    
    def _deal_devicelist(self, lines):
        '''deal adb devices list'''
        devices = []
        for line in lines:
            if line != ""\
            and "List of devices" not in line\
            and '* daemon'not in line and 'offline' not in line:
                line = line.split('\t')
                devices.append({'udid': line[0], 'state': line[1]})
        return devices

    def shell(self, cmd):
        '''
        Execute adb shell command执行ADB SHELL命令
        :Args:
         - cmd -  command
        :Usage:
            shell('dumpsys window')
        '''
        return self.exec_adb('shell ' + cmd)
    
    def cmd(self, args):
        '''adb command, add -s serial by default. return the subprocess.Popen object.'''
        serial = self.serial
        if serial:
            if " " in serial:  # TODO how to include special chars on command line
                serial = "'%s'" % serial
            return self.raw_cmd("-s %s %s" % (serial, args))
        else:
            return self.raw_cmd(args)
        
    def raw_cmd(self, cmd):
        '''adb command. return the subprocess.Popen object.'''
        cmd_line = 'adb ' + cmd
        os.system("adb start-server")
        self.logger.debug("exec {" + cmd_line + "}") 
        return subprocess.Popen(cmd_line, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        

    def push(self, localPath, remotePath):
        '''
        Put file or folder from PC to cellphone.将文件或者文件夹从计算机放入手机
        :Args:
         - localpath -
         - remotepath -
        :Usage:
            push('D:/screen.txt','/storage/sdcard0/360Download/')
        :Return:
            ['344 KB/s (27845 bytes in 0.079s)']
        :Raise:
            - return - ["cannot stat 'D:/scree': No such file or directory"]
        '''
        cmd = 'push -p "%s" "%s"' % (localPath, remotePath)
        return self.exec_adb(cmd)

    def pull(self, srcFile, targetFile):
        '''
        Put file or folder from cellphone to PC.将文件或者文件夹从手机放入计算机
        :Args:
         - srcFile -
         - targetFile -
        :Usage:
            pull('/storage/sdcard0/360Download/new.txt','D:/')
        :Return:
            ['344 KB/s (27845 bytes in 0.079s)']
        :Raises:
            - return - ["cannot stat 'D:/scree': No such file or directory"]
        '''
        cmd = 'pull -p "%s" "%s"' % (srcFile, targetFile)
        return self.exec_adb(cmd)

    def install(self, apk, arg):
        '''
        Install the app.安装手机应用
        :Args:
         - apk - file.apk
        :Usage:
            install('d:/csdn.apk')
        :Return:
            ['Success']
        :Raises:
            - return - ['Invalid APK file: aaaa.apk'],['Missing APK file'],etc
        '''
        if arg:
            cmd='install ' +arg+ ' "' + apk + '"'
        else:
            cmd='install "' + apk + '"'
        return self.exec_adb(cmd)
    
    def installrt(self, apk):
        '''
        Install the app.安装手机应用
        :Args:
         - apk - file.apk
        :Usage:
            install('d:/csdn.apk')
        :Return:
            ['Success']
        :Raises:
            - return - ['Invalid APK file: aaaa.apk'],['Missing APK file'],etc
        '''
        return self.exec_adb(cmd='install -r -t "' + apk + '"')

    def uninstallApk(self, apk, keep=False):
        '''
        Uninstall the app.卸载手机应用
        :Args:
         - apk - app package
        :Usage:
            uninstallApk('net.csdn.app')
        :Return:
            ['Success']
        :Raises:
            - return - ['Failure [DELETE_FAILED_INTERNAL_ERROR]']
        '''
        self.forceStop(apk)
        if keep == True:
            return self.shell(cmd="pm uninstall -k " + apk) 
        return self.shell(cmd="pm uninstall " + apk)

    def isAppinstalled(self, apk):
        '''
        Whether the app is installed or not.手机应用是否安装
        :Args:
         - apk - app package
        :Usage:
            isAppinstalled('net.csdn.app')
        '''
#         command = '-3' if self.getApiLevel() > 15 else''
#         listpkgcmd = 'pm list packages ' + command + ' ' + apk
        listpkgcmd = 'pm list packages ' + apk
        package = self.shell(listpkgcmd)
        if package:
            if package[0] == 'package:' + apk:
                return True
        return False

    def lock(self):
        '''
        Lock the screen锁屏
        :Usage:
            lock()
        '''
        return self.keyevent(26)

    def unlock(self):
        '''
        Unlock the screen解锁屏
        :Usage:
            unlock()
        '''
        if self.isScreenLocked():
            return self.keyevent(26)

    def isScreenLocked(self):
        '''
        Whether screen is locked or not.是否锁屏
        :Usage:
            isScreenLocked()
        :Return:
            ['Success']
        '''
        cmd = "dumpsys window"
        for line in self.shell(cmd):
            if 'mShowingLockscreen=true' in line or 'mScreenOnFully=false' in line:
                return True
        return False

    def forceStop(self, pkg):
        '''
        Force to exit the app.强制退出手机应用
        :Args:
         - pkg - app package
        :Usage:
            forceStop('net.csdn.app')
        :Return:
            ['Success']
        '''
        cmd = 'am force-stop ' + pkg
        return self.shell(cmd)

    def stopAndClear(self, pkg):
        return self.forceStop(pkg)

    def clear(self, pkg):
        '''
        Clear the app data.清理手机应用数据
        :Args:
         - pkg - app package
        :Usage:
            clear('net.csdn.app')
        :Return:
            ['Success']
        '''
        return self.shell(cmd='pm clear ' + pkg)

    def startActivity(self, pkg):
        '''
        Open the app activities.打开应用活动
        :Args:
         - pkg - App package and activity
        :Usage:
            startActivity(net.csdn.app/net.csdn.app.mainactivity)
        :Return:
            ['Starting: Intent { act=android.intent.action.MAIN cat=[android.intent.category.LAUNCHER] cmp=com.example.uibestpractice/.MainActivity }']
        :Raises:
            'Error' in Returns
        '''
        return self.shell('am start %s' % pkg)

    def getRunningAppInfo(self, package):
        return self.shell(cmd='dumpsys package %s' % package)

    
    def getCurrentApp(self):
        '''
        Get current app activity 获取当前手机应用活动
        :Usage:
            getCurrentApp()
        :Return:
            com.example.uibestpractice/.MainActivity
        '''
        cmd = 'dumpsys window windows'
        for line in self.shell(cmd):
            if 'mCurrentFocus' in line:
                current_info = line[:-1].split(" ")[2]
                if "/" in current_info:
                    return (current_info.split('/')[0],current_info.split('/')[1])
                else:
                    return (current_info.split('/')[0],None)
                    

    def hideKeyboard(self):
        '''
        Hide the keyboard.隐藏手机软键盘
        :Usage:
            hideKeyboard()
        '''
        cmd = 'dumpsys input_method'
        for line in self.shell(cmd):
            if 'mWindowVisible=true mWindowWasVisible=true' in line:
                self.back()
                return self.hideKeyboard()
            elif 'mWindowVisible=false mWindowWasVisible=false' in line:
                return 'keyboard is not visible'
        return 'Unknow Error'

    def shake(self):
        '''Shake cellphone手机震动'''
        pass  # 暂未实现

    def takeScreenshot(self, path=None):
        '''
        Take screen shot and return picture or base64code 截图并获取图片或返回base码
        :Args:
         - path - where you will put picture or base64code in
        :Usage:
            takeScreenshot('/storage/sdcard0/')
        '''
        png = "/data/local/tmp/screenshot.png";
        cmd = " ".join(['/system/bin/rm', png + ';', '/system/bin/screencap -p',
                        png]);
        self.shell('"' + cmd + '"')
        if not path:
            tmpScreenshot = os.path.join(tempfile.gettempdir(), "easyuiautomator.png")
            if os.path.exists(tmpScreenshot):
                os.remove(tmpScreenshot)
            self.pull(png, tmpScreenshot)
            tmp = ""
            for line in open(tmpScreenshot, 'rb').readlines():
                tmp += line
            return base64.b64encode(tmp)
        else:
            self.pull(png, path)

    def uiautoTakeshot(self, path=None):
        png = "/data/local/tmp/screenshot.png";
        tmpScreenshot = os.path.join(tempfile.gettempdir(), "easyuiautomator.png")
        if os.path.exists(tmpScreenshot):
            os.remove(tmpScreenshot)
        if path is None:
            self.pull(png, tmpScreenshot)
        else:
            self.pull(path, tmpScreenshot)
        tmp = ""
        for line in open(tmpScreenshot, 'rb').readlines():
            tmp += line
        return base64.b64encode(tmp)


    def background(self, tms):
        '''
        Back to home idle and return to app activity.返回到HOME界面并随后回到应用界面
        :Args:
         - tms - seconds
        :Usage:
            background(1)
        '''
        cmd = 'dumpsys window windows'
        for line in self.shell(cmd):
            if 'name' in line and '/' in line:
                activity = line.split('name=')[1][:-1]
        self.keyevent(KeyCode.KEYCODE_HOME)
        time.sleep(tms)
        return self.startActivity(activity)

    def reset(self, pkg):
        '''
        Reset app by clear it.清理应用数据重置应用
        :Args:
         - pkg - app package
        :Usage:
            reset('net.csdn.app')
        '''
        self.forceStop(pkg)
        return self.clear(pkg)

    def isExistFile(self, path):
        '''
        Whether file is exist.
                          判断是否文件存在
        :Args:
         - path - filepath
        :Usage:
            isExistFile('storage/sdcard0/')
        '''
        if "." in path and "/" in path:
            file_name = os.path.basename(path)
            dir_name = os.path.dirname(path)
            dirs = self.shell("cd %s ; ls" % dir_name)
            if file_name in dirs:
                return True
            else:
                return False
        else:
            dirs = self.shell("cd %s" % path)
            if dirs == []:
                return True
            else:
                return False
            
    def forward(self, system_port, device_port):
        '''Switch port.转换端口'''
        cmd = "forward tcp:" + str(system_port) + " tcp:" + str(device_port)
        return self.exec_adb(cmd)
    
    def forwardlist(self):
        '''view connect port list'''
        cmd = "forward --list"
        lines = self.exec_adb(cmd)
        return [line.strip().split() for line in lines]

    def forwardAbstractPort(self, system_port, device_port):
        self.exec_adb(cmd="forward tcp:" + system_port + " localabstract:" + device_port)

    def _execNoReturn(self, cmd):
        '''No return execute command.无返回执行命令'''
        os.system(cmd)

    def isDeviceConnected(self):
        devices = self.getConnectedDevices()
        if len(devices) > 0:
            return True
        return False

    def broadcast(self, intent):
        self.shell(cmd="am broadcast -a " + intent)

    def broadcastAirplaneMode(self, on):
        cmd = 'am broadcast -a android.intent.action.AIRPLANE_MODE --ez state '
        + (True if on == 1 else False)
        self.shell(cmd)
    
    def setAirplaneMode(self,on):
        self.shell(cmd= 'settings get global airplane_mode_on ' + on)

    def isAirplaneModeOn(self):
        result = self.shell(cmd='settings get global airplane_mode_on')
        try:
            if int(result[0]) == 1:
                return True
            else:
                return False
        except:
            pass

    def isWifiOn(self):
        result = self.shell(cmd='settings get global wifi_on')
        try:
            if int(result[0]) != 0:
                return True
            else:
                return False
        except:
            pass

    def setWifi(self, on):  # on= 'on'  or  on = 'off'
        cmd = 'am start -n io.appium.settings/.Settings -e wifi ' + on;
        self.shell(cmd)

    def isDataOn(self):
        result = self.shell(cmd='settings get global mobile_data')
        try:
            if int(result[0]) == 0:
                return False
            else:
                return True
        except:
            pass

    def setData(self, on):
        cmd = 'am start -n io.appium.settings/.Settings -e data ' + on
        self.shell(cmd)
    
    def availableIMEs(self):
        cmd = "ime list -s"
        return self.shell(cmd)

    def defaultIME(self):
        return self.shell(cmd='settings get secure default_input_method')

    def enableIME(self, imeId):
        self.shell(cmd='ime enable ' + imeId)

    def disableIME(self, imeId):
        self.shell(cmd='ime disable ' + imeId)

    def setIME(self, imeId):
        self.shell(cmd='ime set ' + imeId)

    def killProcessesByName(self, name):
        pids = self.getPidsByName(name)
        for pid in pids:
            self.killProcessByPid(pid)

    def killProcessByPid(self, pid):
        cmd = "kill -9 " + pid
        self.shell(cmd)

    def getPidsByName(self, name):
        cmd = "ps"
        pids = []
        result = self.shell(cmd)
        for line in result:
            if line.endswith(name):
                pids.append(re.split("\s+", line)[1])
        return pids
   
    def back(self):
        return self.keyevent(KeyCode.KEYCODE_BACK)

    def goToHome(self):
        return self.keyevent(KeyCode.KEYCODE_HOME)
    
    def delFile(self, filepath):
        return self.shell(cmd='rm -rf ' + filepath)

    def mkdir(self, remotePath):
        return self.shell(cmd='mkdir -p ' + remotePath)

    def installRemote(self, remoteApk):
        return self.shell(cmd='pm install -r ' + remoteApk)

    def keyevent(self, keycode):
        cmd = 'input keyevent ' + str(keycode)
        return self.shell(cmd)
    
    def press(self, keycode):
        cmd = 'input keyevent ' + str(keycode)
        return self.shell(cmd)
    
    def longPress(self, keycode):
        cmd = 'input keyevent --longpress ' + str(keycode)
        return self.shell(cmd)

    def getApiLevel(self):  
        sdk_version = 0
        try:
            sdk_version = int(self.shell('getprop ro.build.version.sdk')[0])
        except:
            pass
        return sdk_version

    def get_battery(self):
        for info in self.shell(cmd='dumpsys battery'):
            if info.strip().startswith("level:"):
                return info.replace("level:","").strip()
            
    def isOnline(self):
        '''检查设备是否在线'''
        flag = self.adbDevices()
        if flag == "offline":
            return False
        return flag
    
    def getVersionCode(self,packageName):
        versionCode = ""
        for line in self.shell(cmd="dumpsys package " + packageName):
            tmp = line.strip()
            if tmp.startswith("versionCode="):
                versionCode = tmp.split(" ")[0].split("=")[1]
                break
        return 0 if versionCode == "" else int(versionCode)
    
    def putSystemSetting(self,params):
        opt={
             'key':params['key'],
             'value':params['value']
             }
        self.shell("settings put system %s %s"%(opt['key'],opt['value']))
    
    def getSystemSetting(self,key):
        return self.shell("settings get system %s"%key)
    
    def stop_third_app(self, ignore_filter=[]):
        ignore_filter_target = ['com.github.uiautomator','com.github.uiautomator.test','com.testguard.uiautomator2','com.testguard.uiautomator2.test']
        ignore_filter_target += ignore_filter
        cmd = 'pm list package -3'
        for line in self.shell(cmd):
            if 'package:' in line:
                package_name = line[len('package:'):]
                if not package_name in ignore_filter_target:
                    self.forceStop(package_name)
                    
                
#-------------------------------SettingAction START-----------------------------------        
    def settingAction(self,SettingCode):
        cmd = 'am start -a '+SettingCode+' -f 0x10000000'
        self.shell(cmd)
    
    def intoWifiPage(self):
        self.settingAction(SettingAction.ACTION_WIFI_SETTINGS)
    
    def intoBtPage(self):
        self.settingAction(SettingAction.ACTION_BLUETOOTH_SETTINGS)
    
    def intoDataPage(self):
        self.settingAction(SettingAction.ACTION_DATA_ROAMING_SETTINGS)
        
    def intoAirplanePage(self):
        self.settingAction(SettingAction.ACTION_AIRPLANE_MODE_SETTINGS)
        
    def intoGpsPage(self):
        self.settingAction(SettingAction.ACTION_LOCATION_SOURCE_SETTINGS)
    
    def intoNfcPage(self): 
        self.settingAction(SettingAction.ACTION_NFC_SETTINGS)
     
    def intoHotpotPage(self):
        self.settingAction(SettingAction.ACTION_WIRELESS_SETTINGS)
        
    def intoDisplayPage(self):
        self.settingAction(SettingAction.ACTION_DISPLAY_SETTINGS)
        
    def intoSoundPage(self):
        self.settingAction(SettingAction.ACTION_SOUND_SETTINGS)
    
    def setScreenLightMode(self,mode):
        '''设置屏幕亮度模式'''
        opt={
             'key':'screen_brightness_mode',
             'value':mode
             }
        self.putSystemSetting(opt)
    
    def setScreenOff(self,time):
        '''设置屏幕休眠时间 单位s'''
        opt={
             'key':'screen_off_timeout',
             'value':time * 1000
             }
        self.putSystemSetting(opt)
#----------------------------------SettingAction END-----------------------------------        

if __name__ == '__main__':
    a = ADB()
    m =  a.getCurrentApp()
    print m[0],m[1]
    
   

  
    



# coding=utf-8
'''
Created on 2016年4月15日
@author: fengbo
'''

class ErrorCode(object):
    '''
    Error Code.错误码
    '''
    DEVICE_EXECEPTION = 9
    POWER_MONITOR_STARTUP = 18
    COMMAND_TIME_OUT = 21
    SOCKET_CONNECT_FAIL = 10
    SUCCESS = 0
    FAIL = 1 


# coding=utf-8

from easyuiautomator.driver.driver import Driver
import unittest
from easyuiautomator.driver.common.keyevent import KeyCode
from easyuiautomator.driver.common.touch_action import TouchAction
from easyuiautomator.driver.common.multi_action import MultiAction
import time
app = Driver.connect_device()

class TestDriver(unittest.TestCase):
    
    def test_003_get_page_source(self):
        print app.get_device_size()
        print app.get_page_source()
        print app.get_current_activity()
        print app.getOrientation()

    def test_004_press_keycode(self):
        print app.longpress_keycode(KeyCode.KEYCODE_MENU)
        print app.back()

    def test_touch_action(self):
        el = app.find_element_by_id("com.asus.gallery:id/gl_root_view")
        action = TouchAction(app)
        action.press(el).wait(1) \
            .moveTo(x=563, y=895).wait(1) \
            .moveTo(x=563, y=1211).wait(1) \
            .moveTo(x=563, y=1508).wait(1) \
            .release().perform()

    def test_006_multi_action(self):
        action1 = TouchAction(app)
        action1.press(x=243, y=500).wait(100) \
            .moveTo(x=563, y=895).wait(100) \
            .moveTo(x=563, y=1211).wait(100) \
            .moveTo(x=563, y=1508).wait(100) \
            .release()

        action2 = TouchAction(app)
        action2.press(x=500, y=600).wait(1) \
            .moveTo(x=563, y=895).wait(100) \
            .moveTo(x=563, y=1211).wait(100) \
            .moveTo(x=563, y=1508).wait(100) \
            .release()
        mm = MultiAction(app)
        mm.add(action1, action2).perform()

    def test_007_common(self):
        app.wake()  # 唤醒屏幕
        app.compressed_layout_hierarchy()  # 开启uiautomator压缩模式
        app.open_notification()  # 打开通知栏
        print app.get_device_size()
        app.get_screenshot_as_file(r"d:\123.png")

    def test_008_adb_action(self):
        print app.is_app_installed('com.example.uibestpractice')
        print app.start_activity('com.example.uibestpractice', '.MainActivity')
        print app.get_current_activity()
        print app.background(2)
        print app.lock()
        app.wake()
        print app.install_app('C:/Users/fengbo/Downloads/csdn.apk')
        print app.launch_app('net.csdn.csdnplus', '.activity.SwitchLoginActivity')
        print app.close_app('net.csdn.csdnplus')
        print app.reset('net.csdn.csdnplus')
        print app.remove_app('net.csdn.csdnplus')
        print app.push_file('D:/current.txt', 'storage/sdcard0/screen.txt')
        print app.pull_file('storage/sdcard0/360Download/new2.txt', 'D:/')
        print app.hide_keyboard()

    def test_010_zoom_pinch(self):
        ele = app.find_element_by_id("com.asus.gallery:id/gallery_root")
        app.zoom(ele)
        time.sleep(2)
        app.pinch(ele)

    def test_011_tap(self):
        app.tap([(152, 1307), (118, 1307), (520, 1307)])
        
    def test_012_setText(self):
        ele = app.find_element_by_id('com.yep.test.tools:id/edit_phone')
        ele.clear()

    def test_013_get_current_activity(self):
        print app.get_current_activity

    def test_014_scrollTo(self):
        el = app.find_element_by_class_name("android.widget.ListView")
        app.scrollTo(el, "106906621653", 'vertical')

    def test_015_wake_up(self):
        app.wake()
        app.close_app('')
    
    def test_016_find_imag_position(self):
        print app.find_img_position('res/dump_1.png', 'res/dump_2.png')





# coding=utf-8

"""
The By implementation.
"""

class By(object):
    """
    Set of supported locator strategies.
    """

    ID = "id"
    NAME = "name"
    XPATH = "xpath"
    CLASS_NAME = "class name"
    ANDROID_UIAUTOMATOR = '-android uiautomator'
    ACCESSIBILITY_ID = 'accessibility id'
    LINK_TEXT = "link text"
    PARTIAL_LINK_TEXT = "partial link text"

    
    @classmethod
    def is_valid(cls, by):
        for attr in dir(cls):
            if by == getattr(cls, attr):
                return True
        return False


#coding=utf-8
'''
Created on 2017年8月3日

@author: Administrator
'''
import ftplib, os, shutil

from easyuiautomator.common.log import Logger


HOST = "172.16.15.130"
class FtpDownLoad(object):
    
    def __init__(self,host=HOST):
        self.host = host
        self.ftp = ftplib.FTP()
        self.ftp.set_pasv(0) #0 主动模式 1 #被动模式
#         self.ftp.set_debuglevel(2)
        self.bufsize = 1024  #设置缓冲块大小 
        self.logger = Logger.getLogger()

    def _login(self):
        '''登录服务器'''
        try:
            self.ftp.connect(self.host)
            self.ftp.login()
            return True
        except:
            return False
    
    def _rmDir(self,dirname):
        '''删除整个目录结构'''
        try:
            shutil.rmtree(dirname)
        except:
            pass
        
    def downLoadDir(self, remotedir, localdir='.'):
        '''下载远程目录'''
        if self._login():
            path = os.path.join(os.path.abspath(localdir), os.path.basename(remotedir))
            if os.path.exists(path):
                self._rmDir(path)
            os.chdir(os.path.abspath(localdir))
            self._downLoadDir(remotedir)
            os.chdir(os.path.abspath(localdir))
            self.close()
            return True
        else:
            return False
        
    def _downLoadDir(self, remotedir):
        '''下载远程目录'''
        rdir = os.path.basename(remotedir)
        os.makedirs(rdir)
        os.chdir(rdir) 
        self.ftp.cwd(remotedir)
        filelines = []  
        self.ftp.dir(filelines.append)  
        filelines_bk = self.ftp.nlst()
        i = 0
        for file in filelines:
            if 'd' in file.split()[0]: 
                self._downLoadDir(filelines_bk[i].decode("utf-8"))
                self.ftp.cwd('..')  
                os.chdir('..')  
            else:
                newFileName = filelines_bk[i].decode("utf-8")
                fd = open(newFileName, 'wb')  
                self.ftp.retrbinary('RETR %s'%filelines_bk[i], fd.write)  
                fd.close() 
                self.logger.debug(newFileName)
            i += 1
        
     
    def downLoadFile(self, remote_file, local_file='.'):
        '''下载单个文件'''
        if self._login():
            if os.path.isdir(local_file):
                local_file = os.path.join(local_file, os.path.basename(remote_file)) 
            file_handler = open(local_file, 'wb')
            try:
                self.ftp.retrbinary('RETR %s'%(remote_file), file_handler.write)  
            except:
                pass
            finally:
                if file_handler:
                    file_handler.close()
                self.close()
                self.logger.debug(remote_file)
            if os.path.getsize(local_file) > 0:
                return True
            else:
                os.remove(local_file)
                return False
        return False   
        
    def close(self):
        '''关闭ftp'''
        if self.ftp:
            self.ftp.quit()
            self.ftp = None
    
    def test(self):
        if self._login():
            self.ftp.cwd("test/我是中国人")
        
if __name__ == '__main__':
    a = FtpDownLoad()
    print a.downLoadDir("ageingtest", "d://tmp//", )
    a.close()
    


# coding=utf-8
"""
Created on 2015年11月5日

@author: thomas.ning
"""
import copy
from easyuiautomator.driver.common.mobilecommand import MobileCommand as Command


class TouchAction(object):
    """
    used to complex command
    """
    def __init__(self, driver=None):
        self._driver = driver
        self._actions = []
        self._id = None
        self._x = 0.0
        self._y = 0.0

    def tap(self, element=None, x=None, y=None, count=1):
        """
        Perform a tap action on the element

        :Args:
         - element - the element to tap
         - x - (optional) x coordinate to tap, relative to the top left corner of the element.
         - y - (optional) y coordinate. If y is used, x must also be set, and vice versa

        :Usage:
        """
        opts = self._get_opts(element, x, y)
        opts['count'] = count
        self._add_action('tap', opts)
        return self

    def press(self, el=None, x=None, y=None):
        """
        Begin a chain with a press down action at a particular element or point
        """
        self._add_action('press', self._get_opts(el, x, y))
        return self

    def long_press(self, el=None, x=None, y=None, duration=1000):
        """
        Begin a chain with a press down that lasts `duration` milliseconds
        """
        self._add_action('longPress', self._get_opts(el, x, y, duration))
        return self

    def wait(self, ms=0):
        """
        Pause for `ms` milliseconds.
        """
        if ms is None:
            ms = 0
        opts = {'ms': ms}
        self._add_action('wait', opts)
        return self

    def move_to(self, el=None, x=None, y=None):
        """
        Move the pointer from the previous point to the element or point specified
        """
        self._add_action('moveTo', self._get_opts(el, x, y))
        return self

    def release(self):
        """
        End the action by lifting the pointer off the screen
        """
        self._add_action('release', {'id':self._id,'x':self._x, 'y':self._y})
        return self

    def perform(self):
        """
        Perform the action by sending the commands to the server to be operated upon
        """
        params = {'actions': self._actions}
        self._driver._execute(Command.TOUCH_ACTION, params)
        # get rid of actions so the object can be reused
        self._actions = []
        return self

    @property
    def json_wire_gestures(self):
        gestures = []
        for action in self._actions:
            gestures.append(copy.deepcopy(action))
        return gestures

    def _add_action(self, action, options):
        gesture = {
            'action': action,
            'options': options,
        }
        self._actions.append(gesture)

    def _get_opts(self, element, x, y, duration=None):
        opts = {}
        if element is not None:
            opts['id'] = element.id
            self._id = element.id
        # it makes no sense to have x but no y, or vice versa.
        if x is None or y is None:
            x, y = None, None
        opts['x'] = x
        opts['y'] = y
        self._x = x
        self._y = y
        if duration is not None:
            opts['duration'] = duration
        return opts



try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

setup(
    name='easyuiautomator',
    version='1.7',
    platforms='any',
    url='',
    license='MIT',
    author='thomas.ning',
    author_email='ningruhu@163.com',
    description='',
    keywords=('testing', 'android', 'uiautomator'),
    packages=['easyuiautomator', 
              'easyuiautomator.common', 
              'easyuiautomator.driver', 
              'easyuiautomator.driver.common',
              'easyuiautomator.driver.executor'
    ],
    package_data={
        'easyuiautomator': [
            'resource/u1/YepTelecomBootstrap.jar',
            'resource/u2/testguard-uiautomator.apk',
            'resource/u2/testguard-uiautomator-test.apk',
        ],
    },
    data_files=[
                ("Lib/site-packages",
                  ["cv2.pyd"]
                )
    ],
    include_package_data=True,
    install_requires=['requests>=2.18.1'],
    zip_safe=False
)



# coding=utf-8

'''
Created on 2015年11月5日

@author: thomas.ning
'''
import base64,tempfile,os,sys
from .common.by import By
from easyuiautomator.common.exceptions import InvalidSelectorException
from easyuiautomator.common.exceptions import DeviceNotFound
from easyuiautomator.driver.common.mobilecommand import MobileCommand as Command
from easyuiautomator.common.log import Logger
from easyuiautomator.driver.executor.utils import Verify
from easyuiautomator.driver.common.touch_action import TouchAction
from easyuiautomator.driver.executor.utils import ImageUtil


# 收集异常信息
def collectException(func):
    def wrap(*args, **argkw):
        try:
            return func(*args, **argkw)
        except DeviceNotFound, e:
            raise DeviceNotFound(e)
        except:
            msg = sys.exc_info()[1].message
            Logger.getLogger().warning("%s is failed [%s]" % (func.__name__, msg))  
            if 'ignoreExp' not in  argkw.keys():
                raise
            if not argkw['ignoreExp']:
                raise  
    return wrap

class Element(object):
    '''
    All actions of element.

    - element - a element you get by find_element in driver.
    '''

    def __init__(self, parent, id_):
        self._parent = parent
        self._id = id_
        
    @collectException
    def find_element(self, by=By.ID, value=None, thinkTime=None, timeOut=None, ignoreExp=False):
        '''
        Find element by parent element.

        :Args:
         - by - parent element
         - value - text

        :Usage:
            element.find_element(parent_element)
        '''
        if not By.is_valid(by) or not isinstance(value, str):
            raise InvalidSelectorException("Invalid locator values passed in")
        if thinkTime is not None :
            thinkTime = float(thinkTime)
        if timeOut is not None:
            timeOut = float(timeOut)
        return self._execute(Command.FINE_CHILD_ELEMNT,
                             {'using': by, 
                              'value': value, 
                              'thinkTime':thinkTime, 
                              'timeOut':timeOut})['value']
    @collectException
    def find_elements(self, by=By.ID, value=None, thinkTime=None, timeOut=None, ignoreExp=False):
        '''
        Find elements by parent element.

        :Args:
         - by - parent element
         - value - text

        :Usage:
            element.find_elements(parent_element)
        '''
        if not By.is_valid(by) or not isinstance(value, str):
            raise InvalidSelectorException("Invalid locator values passed in")
        if thinkTime is not None :
            thinkTime = float(thinkTime)
        if timeOut is not None:
            timeOut = float(timeOut)
        return self._execute(Command.FINE_CHILD_ELEMNTS,
                             {'using': by, 
                              'value': value, 
                              'thinkTime':thinkTime, 
                              'timeOut':timeOut})['value']

    def find_element_by_id(self, id_, thinkTime=None, timeOut=None, ignoreExp=False):
        """Finds an element by id.

        :Args:
         - id\_ - The id of the element to be found.

        :Usage:
            element.find_element_by_id('foo')
        """
        return self.find_element(by=By.ID, value=id_, thinkTime=thinkTime, timeOut=timeOut, ignoreExp=ignoreExp)

    def find_elements_by_id(self, id_, thinkTime=None, timeOut=None, ignoreExp=False):
        """
        Finds multiple elements by id.

        :Args:
         - id\_ - The id of the elements to be found.

        :Usage:
            element.find_elements_by_id('foo')
        """
        return self.find_elements(by=By.ID, value=id_, thinkTime=thinkTime, timeOut=timeOut, ignoreExp=ignoreExp)

    def find_element_by_xpath(self, xpath, thinkTime=None, timeOut=None, ignoreExp=False):
        """
        Finds an element by xpath.

        :Args:
         - xpath - The xpath locator of the element to find.

        :Usage:
            element.find_element_by_xpath('//div/td[1]')
        """
        return self.find_element(by=By.XPATH, value=xpath, thinkTime=thinkTime, timeOut=timeOut, ignoreExp=ignoreExp)

    def find_elements_by_xpath(self, xpath, thinkTime=None, timeOut=None, ignoreExp=False):
        """
        Finds multiple elements by xpath.

        :Args:
         - xpath - The xpath locator of the elements to be found.

        :Usage:
            element.find_elements_by_xpath("//div[contains(@class, 'foo')]")
        """
        return self.find_elements(by=By.XPATH, value=xpath, thinkTime=thinkTime, timeOut=timeOut, ignoreExp=ignoreExp)

    def find_element_by_name(self, name, thinkTime=None, timeOut=None, ignoreExp=False):
        """
        Finds an element by name.

        :Args:
         - name: The name of the element to find.

        :Usage:
            element.find_element_by_name('foo')
        """
        return self.find_element(by=By.NAME, value=name, thinkTime=thinkTime, timeOut=timeOut, ignoreExp=ignoreExp)

    def find_elements_by_name(self, name, thinkTime=None, timeOut=None, ignoreExp=False):
        """
        Finds elements by name.

        :Args:
         - name: The name of the elements to find.

        :Usage:
            element.find_elements_by_name('foo')
        """
        return self.find_elements(by=By.NAME, value=name, thinkTime=thinkTime, timeOut=timeOut, ignoreExp=ignoreExp)

    def find_element_by_class_name(self, name, thinkTime=None, timeOut=None, ignoreExp=False):
        """
        Finds an element by class name.

        :Args:
         - name: The class name of the element to find.

        :Usage:
            element.find_element_by_class_name('foo')
        """
        return self.find_element(by=By.CLASS_NAME, value=name, thinkTime=thinkTime, timeOut=timeOut, ignoreExp=ignoreExp)

    def find_elements_by_class_name(self, name, thinkTime=None, timeOut=None, ignoreExp=False):
        """
        Finds elements by class name.

        :Args:
         - name: The class name of the elements to find.

        :Usage:
            element.find_elements_by_class_name('foo')
        """
        return self.find_elements(by=By.CLASS_NAME, value=name, thinkTime=thinkTime, timeOut=timeOut, ignoreExp=ignoreExp)

    def find_element_by_uiautomator(self, uia_string, thinkTime=None, timeOut=None, ignoreExp=False):
        """Finds element by uiautomator in Android.

        :Args:
         - uia_string - The element name in the Android UIAutomator library

        :Usage:
            element.find_element_by_android_uiautomator('.elements()[1].cells()[2]')
        """
        return self.find_element(by=By.ANDROID_UIAUTOMATOR, value=uia_string, thinkTime=thinkTime, timeOut=timeOut, ignoreExp=ignoreExp)

    def find_elements_by_uiautomator(self, uia_string, thinkTime=None, timeOut=None, ignoreExp=False):
        """Finds elements by uiautomator in Android.

        :Args:
         - uia_string - The element name in the Android UIAutomator library

        :Usage:
            element.find_elements_by_android_uiautomator('.elements()[1].cells()[2]')
        """
        return self.find_elements(by=By.ANDROID_UIAUTOMATOR, value=uia_string, thinkTime=thinkTime, timeOut=timeOut, ignoreExp=ignoreExp)

    def find_element_by_accessibility_id(self, id_, thinkTime=None, timeOut=None, ignoreExp=False):
        """Finds an element by accessibility id.

        :Args:
         - id - a string corresponding to a recursive element search using the
         Id/Name that the native Accessibility options utilize

        :Usage:
            element.find_element_by_accessibility_id()
        """
        return self.find_element(by=By.ACCESSIBILITY_ID, value=id_, thinkTime=thinkTime, timeOut=timeOut, ignoreExp=ignoreExp)

    def find_elements_by_accessibility_id(self, id_, thinkTime=None, timeOut=None, ignoreExp=False):
        """Finds elements by accessibility id.

        :Args:
         - id - a string corresponding to a recursive element search using the
         Id/Name that the native Accessibility options utilize

        :Usage:
            element.find_elements_by_accessibility_id()
        """
        return self.find_elements(by=By.ACCESSIBILITY_ID, value=id_, thinkTime=thinkTime, timeOut=timeOut, ignoreExp=ignoreExp)
    
    def _execute(self, command, params=None):

        if params is None:
            params = {}
        params['id'] = self._id
        return self._parent._execute(command, params)

    def click(self):
        """Clicks the element.

        :Usage:
            element.click()
        """
        self._execute(Command.CLICK)

    def longClick(self, duration=None):
        """long Clicks the element.

        :Usage:
            element.longClick(duration)
        """
        opts = {
                'duration':duration
                }
        self._execute(Command.LONG_CLICK, opts)

    def setText(self, value, replace=True):
        '''
        Set Text in textedit.

        :Args:
          - value - text you will set

        :Usage:
            element.setText()
        '''
        opts = {
            'text': value,
            'replace':replace
        }
        return self._execute(Command.SET_TEXT, opts)

    def getText(self):
        '''
        Get text from element attribute.

        :Usage:
            element.getText()
        '''
        return self._getAttribute("text")

    def getResourceId(self):
        '''
        Get resource id from element attribute.

        :Usage:
            element.getResourceId()
        '''
        return self._getAttribute("resourceId")
    
    def getBounds(self):
        '''
        Get bounds from element attribute.

        :Usage:
            element.getResourceId()
        '''
        return self._getAttribute("bounds")

    def getClassName(self):
        '''
        Get class name from element attribute.

        :Usage:
            element.getClassName()
        '''
        return self._getAttribute("className")

    def getContent_Desc(self):
        '''
        Get content-desc element atribute

        :Usage:
            element.getContent()
        '''
        return self._getAttribute("content_desc")
    
    def getParent(self):
        '''
        Get content-desc element atribute

        :Usage:
            element.getContent()
        '''
        return self._execute(Command.GET_PARENT)['value']

    def isChecked(self):
        '''
        Check whether element is checked.

        :Usage:
            element.isChecked()

        :Return:
            true or false
        '''
        return self._getAttribute("checked")

    def isCheckable(self):
        '''
        Check whether element is checkable.

        :Usage:
            element.isCheckable()
        '''
        return self._getAttribute("checkable")

    def isClickable(self):
        '''
        Check whether element is clickable.

        :Usage:
            element.isClickable()
        '''
        return self._getAttribute("clickable")

    def isEnabled(self):
        '''
        Check whether element is enabled.

        :Usage:
            element.isEnabled()
        '''
        return self._getAttribute("enabled")

    def isFocuable(self):
        '''
        Check whether element is focuable.

        :Usage:
            element.isFocuable()
        '''
        return self._getAttribute("focusable")

    def isFocused(self):
        '''
        Check whether element is focused.

        :Usage:
            element.isFocused()
        '''
        return self._getAttribute("focused")

    def isScrollable(self):
        '''
        Check whether element is scrollable.

        :Usage:
            element.isScrollable()
        '''
        return self._getAttribute("scrollable")

    def isLongClickable(self):
        '''
        Check whether element is longclickable.

        :Usage:
            element.isLongClickable()
        '''
        return self._getAttribute("longClickable")

    def isSelected(self):
        '''
        Check whether element is selected.

        :Usage:
            element.isSelected()
        '''
        return self._getAttribute("selected")

    def isDisplayed(self):
        '''
        Check whether element is displayed.

        :Usage:
            element.isDisplayed()
        '''
        return self._getAttribute("displayed")

    def getSize(self):
        '''
        Get size from element attribute.

        :Usage:
            element.getSize()

        :Return:
            length and width
        '''
        return self._execute(Command.GET_SIZE)['value']

    def getLocation(self):
        '''
        Get location from element attribute.

        :Usage:
            element.getLocation()

        :Return:
            x and y of element
        '''
        return self._execute(Command.GET_LOCATION)['value']

    def _getAttribute(self, attribute):
        opts = {
            "attribute": attribute
        }
        return self._execute(Command.GET_ATTRIUBTE, opts)["value"]

    def clear(self):
        '''
        Clear the text in textedit.

        :Usage:
            element.clear()
        '''
        """Clears the text if it's a text entry element."""
        
        self._execute(Command.CLEAR_ELEMENT)

    def get_screenshot_as_file(self, filename,scale=1.0,quality=50):
        """
        Gets the screenshot of the current window. Returns False if there is
           any IOError, else returns True. Use full paths in your filename.

        :Args:
         - filename: The full path you wish to save your screenshot to.

        :Usage:
            element.get_screenshot_as_file('/Screenshots/foo.png')
        """
        png = self.get_screenshot_as_png(scale,quality)
        try:
            with open(filename, 'wb') as f:
                f.write(png)
        except IOError:
            return False
        finally:
            del png
        return True

    def get_screenshot_as_png(self,scale=1.0,quality=50):
        """
        Gets the screenshot of the current window as a binary data.
        
        :Args:
         - scale: The screen size scale,default:1.0.
         - quality: The screen quality,default:50%  0-100.
        
        :Usage:
            driver.get_screenshot_as_png(0.8,80%)
        """
        return base64.b64decode(self.get_screenshot_as_base64(scale,quality).encode('ascii'))

    def get_screenshot_as_base64(self,scale=1.0,quality=50):
        """
        Gets the screenshot of the current window as a base64 encoded string
           which is useful in embedded images in HTML.
        
        :Args:
         - scale: The screen size scale,default:1.0.
         - quality: The screen quality,default:50%  0-100.
        
        :Usage:
            driver.get_screenshot_as_base64()
        
        """
#         opts = {
#             "scale": scale,
#             "quality": quality
#         }
        return self._overdo_takeshot()
#         return self._execute(Command.SCREENSHOT, opts)['value']
    
    def _overdo_takeshot(self):
        temp = tempfile.mktemp()
        temp_dest = tempfile.mktemp(suffix=".png")
        if self._parent.get_screenshot_as_file(temp):
            startx, starty, endx, endy = self.getBounds().split(" ")
            ImageUtil.crop(float(startx), float(starty), float(endx), float(endy), temp, temp_dest)
            os.remove(temp)
            tmpContent = ""
            with open(temp_dest, 'rb') as f:
                for line in f.readlines():
                    tmpContent += line
            os.remove(temp_dest)
            return base64.b64encode(tmpContent)
              
    #####################################################################################################
    ############################################   NEW PLUS    ##########################################
    #####################################################################################################
    def dragToEle(self, element, duration=None):
        """
        The Element drag to another element's position..

        :Args:
         - element: The full path you wish to save your screenshot to.
         - duration: The time it costs.

        :Usage:
            element.dragToEle(element)
        """
        opts = {
            'destElId': element.id,
            'duration':duration
        }

        return  self._execute(Command.DRAG, opts)['value']

    def dragToPos(self, x, y, duration=None):
        """
        The Element drag to another element's position by x and y..

        :Args:
         - x: Position x
         - y: Position y
         - duration: The time it costs.

        :Usage:
            element.dragToPos(100,200)
        """
        opts = {
            'endX':x,
            'endY':y,
            'duration':duration
        }
        return  self._execute(Command.DRAG, opts)['value']

    def scroll(self, destination_el):
        """Scrolls from one element to another

        :Args:
         - destinationEl - the element to scroll to

        :Usage:
            element.scroll(el2)
        """
        action = TouchAction(self)
        action.press(self).moveTo(destination_el).release().perform()
        return self
    
    @collectException
    def scrollTo(self, text, direction='vertical'):
        """Scrolls from one element to another

        :Args:
         - text - the text to scroll to
         - direction - vertical horizontal

        :Usage:
            element.scrollTo(text)
        """
        opts = {
            'text': text,
            'direction': direction
        }
        self._execute(Command.SCROLL_TO, opts)
        return True

    def swipeLeft(self, duration=None):
        '''
        swipe from the right border of element to he left border of element with duration

        :Usage:
            element.swipeLeft(duration)
        '''
        return self._swipeOrientation("left", duration)

    def swipeRight(self, duration=None):
        '''
        swipe from the left border of element to he right border of element with duration

        :Usage:
            element.swipeRight(duration)
        '''
        return self._swipeOrientation("right", duration)

    def swipeUp(self, duration=None):
        '''
        swipe from the up border of element to he down border of element with duration

        :Usage:
            element.swipeUp(duration)
        '''
        return self._swipeOrientation("up", duration)

    def swipeDown(self, duration=None):
        '''
        swipe from the up border of element to he down border of element with duration

        :Usage:
            element.swipeDown(duration)
        '''
        return self._swipeOrientation("down", duration)

    def _swipeOrientation(self, direction, duration=None):
        '''
        swipe from the some border of element to he some border of element with duration
        base method,no suggest used
        :Args:
        dirction: left,right,up,down
        duation : from second to steps default 20

        :Usage:
            element.swipeDown(duration)
        '''
        opts = {
                'direction': direction,
                'duration' : duration
                }
        return self._execute(Command.SWIPE_ORIENTAION, opts)['value']

    def pinch(self, percent=200, steps=50):
        """Pinch on an element a certain amount

        :Args:
         - percent - (optional) amount to pinch. Defaults to 200%
         - steps - (optional) number of steps in the pinch action

        :Usage:
            element.pinch(element)
        """
        opts = {
            'percent': percent,
            'steps': steps,
        }
        self._execute(Command.PINCH, opts)
        return self

    def zoom(self, percent=200, steps=50):
        """Zooms in on an element a certain amount

        :Args:
         - element - the element to zoom
         - percent - (optional) amount to zoom. Defaults to 200%
         - steps - (optional) number of steps in the zoom action

        :Usage:
            element.zoom(element)
        """
        opts = {
            'percent': percent,
            'steps': steps,
        }
        self._execute(Command.ZOOM, opts)
        return self

    def verifyText(self, text):
        """
        Verify element's Text of attribute whether is equal to the text you set.

        :Args:
         - text:The text you want to verify.

        :Usage:
            ele.verifyText('Settings')
        """
        return Verify.verifyText(self.getText(), text)

    def verifyNotText(self, text):
        """
        This is the opposite of verifyText.

        :Args:
         - text:The text you want to verify.

        :Usage:
            ele.verifyNotText('Settings')
        """
        return Verify.verifyNotText(self.getText(), text)

    def verifyContent_Desc(self, content_desc):
        """
        Verify element's content_desc of attribute whether is equal to the text you set.

        :Args:
         - content_desc:The content_desc you want to verify.

        :Usage:
            ele.verifyContent("text....")
        """
        return Verify.verifyText(self.getText(), content_desc)

    def verifyNotContent_Desc(self, content_desc):
        """
        This is the opposite of verifyContent_desc.

        :Args:
         - content_desc:The content_desc you want to verify.

        :Usage:
            ele.verifyNotContent("text....")
        """
        return Verify.verifyNotText(self.getText(), content_desc)

    def verifyImage(self, Image, similarity=100):
        """
        Verify element's Image whether is equal to the image you set.

        :Args:
         - Image: The content_desc you want to compare.

        :Usage:
            ele.verifyImage("a/a/a.png.")
        """
        temp=tempfile.mktemp()
        if self.get_screenshot_as_file(temp):
            a=Verify.verifyImage(temp, Image, similarity)
            if os.path.exists(temp):os.remove(temp)
            return a
        else:
            return False

    def verifyTextRe(self, text):
        """
        Verify element's Text of attribute whether is about equal to the text you set.

        :Args:
         - text:The text you want to verify.

        :Usage:
            ele.verifyTextRe('Settings') or (u"邮箱")
        """
        return Verify.verifyTextRe(self.getText(), text)

    def verifyNotTextRe(self, text):
        """
        This is the opposite of verifyTextRe.

        :Args:
         - text:The text you want to verify.

        :Usage:
            ele.verifyNotTextRe('Settings') or (u"邮箱")
        """
        return Verify.verifyNotTextRe(self.getText(), text)

    def verifyContent_DescRe(self, content_desc):
        """
        Verify element's content_desc of attribute whether is equal to the text you set.

        :Args:
         - content_desc:The content_desc you want to verify.

        :Usage:
            ele.verifyContent_DescRe("text....") or (u"邮箱")
        """
        return Verify.verifyTextRe(self.getContent_Desc(), content_desc)

    def verifyNotContent_DescRe(self, content_desc):
        """
        This is the opposite of verifyContent_descRe.

        :Args:
         - content_desc:The content_desc you want to verify.

        :Usage:
            ele.verifyNotContent_DescRe("text....") or (u"邮箱")
        """
        return Verify.verifyNotTextRe(self.getContent_Desc(), content_desc)

    def scrollTop(self,duration=None):
        '''滑动到顶部,例如在电话本，文件管理中
        :Usage:
            ele.scrollTop()
        '''
        opts = {
            'direction': 'top',
            'duration' : duration
        }
        self._execute(Command.SCROLL_ORIENTAION, opts)
        return self

    def scrollBottom(self,duration=None):
        '''滑动到底部，例如在电话本，文件管理中
        API适用于ListView 或 ScrollView 等可滑动的元素

        :Usage:
            ele.scrollBottom()
        '''
        opts = {
            'direction': 'bottom',
            'duration' : duration
        }

        self._execute(Command.SCROLL_ORIENTAION, opts)
        return self

########################################################################
    @property
    def parent(self):
        return self._parent

    @property
    def id(self):
        return self._id

    def __eq__(self, element):
        return hasattr(element, 'id') and self._id == element.id

    def __ne__(self, element):
        return not self.__eq__(element)
